@@ -42,7 +42,7 @@ Transaction::Transaction(const Database& db, Transaction::TransactionType type)
 void Transaction::reset(TransactionType type)
 {
     if (m_txn) {
-        qWarning(ENGINE) << "Resetting a Transaction without calling abort/commit";
+        qCWarning(ENGINE) << "Resetting a Transaction without calling abort/commit";
         abort();
     }
     init(type);
@@ -53,7 +53,7 @@ void Transaction::init(TransactionType type)
     uint flags = type == ReadOnly ? MDB_RDONLY : 0;
     int rc = mdb_txn_begin(m_env, nullptr, flags, &m_txn);
     if (rc) {
-        qCDebug(ENGINE) << "Transaction" << mdb_strerror(rc);
+        qCDebug(ENGINE) << "Transaction init() error:" << rc << mdb_strerror(rc);
         return;
     }
 
@@ -70,7 +70,7 @@ Transaction::Transaction(Database* db, Transaction::TransactionType type)
 Transaction::~Transaction()
 {
     if (m_writeTrans) {
-        qWarning(ENGINE) << "Closing an active WriteTransaction without calling abort/commit";
+        qCWarning(ENGINE) << "Closing an active WriteTransaction without calling abort/commit";
     }
 
     if (m_txn) {
@@ -211,6 +211,16 @@ void Transaction::removePhaseOne(quint64 id)
     contentIndexingDb.del(id);
 }
 
+void Transaction::removeFailed(quint64 id)
+{
+    Q_ASSERT(m_txn);
+    Q_ASSERT(id > 0);
+    Q_ASSERT(m_writeTrans);
+
+    DocumentIdDB failedIdDb(m_dbis.failedIdDbi, m_txn);
+    failedIdDb.del(id);
+}
+
 void Transaction::addFailed(quint64 id)
 {
     Q_ASSERT(m_txn);
@@ -310,7 +320,7 @@ void Transaction::abort()
 // Queries
 //
 
-PostingIterator* Transaction::postingIterator(const EngineQuery& query) const
+PostingIterator::Ptr Transaction::postingIterator(const EngineQuery &query) const
 {
     PostingDB postingDb(m_dbis.postingDbi, m_txn);
     PositionDB positionDb(m_dbis.positionDBi, m_txn);
@@ -336,51 +346,51 @@ PostingIterator* Transaction::postingIterator(const EngineQuery& query) const
             qCDebug(ENGINE) << "Degenerated Phrase with 1 Term:" <<  query;
             return postingIterator(subQueries[0]);
         }
-        QVector<VectorPositionInfoIterator*> vec;
+        std::vector<std::unique_ptr<VectorPositionInfoIterator>> vec;
         vec.reserve(subQueries.size());
         for (const EngineQuery& q : subQueries) {
             if (!q.leaf()) {
                 qCDebug(ENGINE) << "Transaction::toPostingIterator" << "Phrase subqueries must be leafs";
                 continue;
             }
-            auto termMatch = positionDb.iter(q.term());
+            auto termMatch = std::unique_ptr<VectorPositionInfoIterator>(positionDb.iter(q.term()));
             if (!termMatch) {
                 return nullptr;
             }
-            vec << termMatch;
+            vec.push_back(std::move(termMatch));
         }
 
-        return new PhraseAndIterator(vec);
+        return std::make_unique<PhraseAndIterator>(std::move(vec));
     }
 
     return nullptr;
 }
 
-PostingIterator* Transaction::postingCompIterator(const QByteArray& prefix, qlonglong value, PostingDB::Comparator com) const
+PostingIterator::Ptr Transaction::postingCompIterator(const QByteArray &prefix, qlonglong value, PostingDB::Comparator com) const
 {
     PostingDB postingDb(m_dbis.postingDbi, m_txn);
     return postingDb.compIter(prefix, value, com);
 }
 
-PostingIterator* Transaction::postingCompIterator(const QByteArray& prefix, double value, PostingDB::Comparator com) const
+PostingIterator::Ptr Transaction::postingCompIterator(const QByteArray &prefix, double value, PostingDB::Comparator com) const
 {
     PostingDB postingDb(m_dbis.postingDbi, m_txn);
     return postingDb.compIter(prefix, value, com);
 }
 
-PostingIterator* Transaction::postingCompIterator(const QByteArray& prefix, const QByteArray& value, PostingDB::Comparator com) const
+PostingIterator::Ptr Transaction::postingCompIterator(const QByteArray &prefix, const QByteArray &value, PostingDB::Comparator com) const
 {
     PostingDB postingDb(m_dbis.postingDbi, m_txn);
     return postingDb.compIter(prefix, value, com);
 }
 
-PostingIterator* Transaction::mTimeRangeIter(quint32 beginTime, quint32 endTime) const
+PostingIterator::Ptr Transaction::mTimeRangeIter(quint32 beginTime, quint32 endTime) const
 {
     MTimeDB mTimeDb(m_dbis.mtimeDbi, m_txn);
     return mTimeDb.iterRange(beginTime, endTime);
 }
 
-PostingIterator* Transaction::docUrlIter(quint64 id) const
+PostingIterator::Ptr Transaction::docUrlIter(quint64 id) const
 {
     DocumentUrlDB docUrlDb(m_dbis.idTreeDbi, m_dbis.idFilenameDbi, m_txn);
     return docUrlDb.iter(id);
@@ -595,4 +605,3 @@ void Transaction::checkPostingDbinTermsDb()
         }
     }
 }
-
