@@ -1,22 +1,9 @@
 /*
- * This file is part of the KDE Baloo project.
- * Copyright (C) 2015  Vishesh Handa <vhanda@kde.org>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
- */
+    This file is part of the KDE Baloo project.
+    SPDX-FileCopyrightText: 2015 Vishesh Handa <vhanda@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.1-or-later
+*/
 
 #ifndef BALOO_TRANSACTION_H
 #define BALOO_TRANSACTION_H
@@ -27,8 +14,8 @@
 #include "writetransaction.h"
 #include "documenttimedb.h"
 #include <functional>
+#include <memory>
 
-#include <QString>
 #include <lmdb.h>
 
 namespace Baloo {
@@ -45,7 +32,7 @@ class BALOO_ENGINE_EXPORT Transaction
 public:
     enum TransactionType {
         ReadOnly,
-        ReadWrite
+        ReadWrite,
     };
     Transaction(const Database& db, TransactionType type);
     Transaction(Database* db, TransactionType type);
@@ -65,18 +52,16 @@ public:
      * \p path into an id.
      */
     quint64 documentId(const QByteArray& path) const;
-    QVector<quint64> childrenDocumentId(quint64 parentId) const;
     QByteArray documentData(quint64 id) const;
 
     DocumentTimeDB::TimeInfo documentTimeInfo(quint64 id) const;
 
-    QVector<quint64> exec(const EngineQuery& query, int limit = -1) const;
-
-    PostingIterator* postingIterator(const EngineQuery& query) const;
-    PostingIterator* postingCompIterator(const QByteArray& prefix, qlonglong value, PostingDB::Comparator com) const;
-    PostingIterator* mTimeIter(quint32 mtime, MTimeDB::Comparator com) const;
-    PostingIterator* mTimeRangeIter(quint32 beginTime, quint32 endTime) const;
-    PostingIterator* docUrlIter(quint64 id) const;
+    PostingIterator::Ptr postingIterator(const EngineQuery &query) const;
+    PostingIterator::Ptr postingCompIterator(const QByteArray &prefix, qlonglong value, PostingDB::Comparator com) const;
+    PostingIterator::Ptr postingCompIterator(const QByteArray &prefix, double value, PostingDB::Comparator com) const;
+    PostingIterator::Ptr postingCompIterator(const QByteArray &prefix, const QByteArray &value, PostingDB::Comparator com) const;
+    PostingIterator::Ptr mTimeRangeIter(quint32 beginTime, quint32 endTime) const;
+    PostingIterator::Ptr docUrlIter(quint64 id) const;
 
     QVector<quint64> fetchPhaseOneIds(int size) const;
     uint phaseOneSize() const;
@@ -96,9 +81,9 @@ public:
     //
     // Transaction handling
     //
-    void commit();
+    bool commit();
     void abort();
-    bool hasChanges() const;
+    void reset(TransactionType type);
 
     //
     // Write Methods
@@ -107,8 +92,9 @@ public:
     void removeDocument(quint64 id);
     void removeRecursively(quint64 parentId);
     void addFailed(quint64 id);
+    void removeFailed(quint64 id);
 
-    bool removeRecursively(quint64 parentId, std::function<bool(quint64)> shouldDelete)
+    bool removeRecursively(quint64 parentId, std::function<bool(const QByteArray &)> shouldDelete)
     {
         Q_ASSERT(m_txn);
         Q_ASSERT(m_writeTrans);
@@ -127,11 +113,12 @@ public:
 
 private:
     Transaction(const Transaction& rhs) = delete;
+    void init(TransactionType type);
 
     const DatabaseDbis& m_dbis;
     MDB_txn *m_txn = nullptr;
     MDB_env *m_env = nullptr;
-    WriteTransaction *m_writeTrans = nullptr;
+    std::unique_ptr<WriteTransaction> m_writeTrans;
 
     friend class DatabaseSanitizerImpl;
     friend class DBState; // for testing
