@@ -1,25 +1,14 @@
 /*
- * Copyright (C) 2014-2015  Vishesh Handa <vhanda@kde.org>
- * Copyright (C) 2014  Denis Steckelmacher <steckdenis@yahoo.fr>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
- */
+    SPDX-FileCopyrightText: 2014-2015 Vishesh Handa <vhanda@kde.org>
+    SPDX-FileCopyrightText: 2014 Denis Steckelmacher <steckdenis@yahoo.fr>
+
+    SPDX-License-Identifier: LGPL-2.1-or-later
+*/
 
 #include "advancedqueryparser.h"
 
+#include "baloodebug.h"
+
 #include <QStringList>
 #include <QStack>
 #include <QDate>
@@ -41,6 +30,10 @@ static QStringList lex(const QString& text)
 
         if (c == QLatin1Char('"')) {
             // Quotes start or end string literals
+            if (inQuotes) {
+                tokenList.append(token);
+                token.clear();
+            }
             inQuotes = !inQuotes;
         } else if (inQuotes) {
             // Don't do any processing in strings
@@ -51,21 +44,21 @@ static QStringList lex(const QString& text)
                 tokenList.append(token);
                 token.clear();
             }
-        } else if (c == '(' || c == ')') {
+        } else if (c == QLatin1Char('(') || c == QLatin1Char(')')) {
             // Parentheses end tokens, and are tokens by themselves
             if (!token.isEmpty()) {
                 tokenList.append(token);
                 token.clear();
             }
             tokenList.append(c);
-        } else if (c == '>' || c == '<' || c == ':' || c == '=') {
+        } else if (c == QLatin1Char('>') || c == QLatin1Char('<') || c == QLatin1Char(':') || c == QLatin1Char('=')) {
             // Operators end tokens
             if (!token.isEmpty()) {
                 tokenList.append(token);
                 token.clear();
             }
             // accept '=' after any of the above
-            if (text.at(i + 1) == '=') {
+            if (((i + 1) < end) && (text.at(i + 1) == QLatin1Char('='))) {
                 tokenList.append(text.mid(i, 2));
                 i++;
             } else {
@@ -88,31 +81,9 @@ static void addTermToStack(QStack<Term>& stack, const Term& termInConstruction,
 {
     Term &tos = stack.top();
 
-    if (tos.isEmpty()) {
-        // Empty top of stack, just assign termInConstruction to it
-        tos = termInConstruction;
-        return;
-    }
-
     tos = Term(tos, op, termInConstruction);
 }
 
-static QVariant tokenToVariant(const QString& token)
-{
-    bool okay = false;
-    int intValue = token.toInt(&okay);
-    if (okay) {
-        return QVariant(intValue);
-    }
-
-    QDate date = QDate::fromString(token, Qt::ISODate);
-    if (date.isValid() && !date.isNull()) {
-        return date;
-    }
-
-    return token;
-}
-
 Term AdvancedQueryParser::parse(const QString& text)
 {
     // The parser does not do any look-ahead but has to store some state
@@ -133,32 +104,25 @@ Term AdvancedQueryParser::parse(const QString& text)
             // termInConstruction so that "foo bar baz" is parsed as expected.
             auto property = termInConstruction.value().toString();
             if (property.isEmpty()) {
-                qDebug() << "Binary operator without first argument encountered:" << text;
+                qCDebug(BALOO) << "Binary operator without first argument encountered:" << text;
                 return Term();
             }
             termInConstruction.setProperty(property);
 
-            QVariant value = tokenToVariant(token);
-            if (value.type() != QVariant::String) {
-                if (termInConstruction.comparator() == Term::Contains) {
-                    termInConstruction.setComparator(Term::Equal);
-                }
-            }
-
-            termInConstruction.setValue(value);
+            termInConstruction.setValue(token);
             valueExpected = false;
             continue;
         }
 
         // Handle the logic operators
-        if (token == QStringLiteral("AND")) {
+        if (token == QLatin1String("AND")) {
             if (!termInConstruction.isEmpty()) {
                 addTermToStack(stack, termInConstruction, ops.top());
                 termInConstruction = Term();
             }
             ops.top() = Term::And;
             continue;
-        } else if (token == QStringLiteral("OR")) {
+        } else if (token == QLatin1String("OR")) {
             if (!termInConstruction.isEmpty()) {
                 addTermToStack(stack, termInConstruction, ops.top());
                 termInConstruction = Term();
@@ -170,7 +134,7 @@ Term AdvancedQueryParser::parse(const QString& text)
         // Handle the different comparators (and braces)
         Term::Comparator comparator = Term::Auto;
 
-        switch (token.at(0).toLatin1()) {
+        switch (token.isEmpty() ? '\0' : token.at(0).toLatin1()) {
             case ':':
                 comparator = Term::Contains;
                 break;
@@ -180,7 +144,7 @@ Term AdvancedQueryParser::parse(const QString& text)
             case '<': {
                 if (token.size() == 1) {
                     comparator = Term::Less;
-                } else if (token[1] == '=') {
+                } else if (token[1] == QLatin1Char('=')) {
                     comparator = Term::LessEqual;
                 }
                 break;
@@ -188,7 +152,7 @@ Term AdvancedQueryParser::parse(const QString& text)
             case '>': {
                 if (token.size() == 1) {
                     comparator = Term::Greater;
-                } else if (token[1] == '=') {
+                } else if (token[1] == QLatin1Char('=')) {
                     comparator = Term::GreaterEqual;
                 }
                 break;
@@ -241,10 +205,23 @@ Term AdvancedQueryParser::parse(const QString& text)
         }
     }
 
+    if (valueExpected) {
+        termInConstruction.setProperty(termInConstruction.value().toString());
+        termInConstruction.setValue(QString());
+        termInConstruction.setComparator(Term::Contains);
+    }
+
     if (termInConstruction.value().isValid()) {
         addTermToStack(stack, termInConstruction, ops.top());
     }
 
+    // Process unclosed parentheses
+    ops.pop();
+    while (stack.size() > 1) {
+	// stack.pop() is the term that has to be closed. Append
+	// it to the term just above it.
+	addTermToStack(stack, stack.pop(), ops.top());
+    }
+
     return stack.top();
 }
-
